/*
*	ebc2asc.c 
*
*	Copyright(c)1992 Sanchez Computer Associates, Inc.
*	All Rights Reserved
*
*	UNIX:	Harsha Lakshmikantha 01 March 2000
*
*	DESC:	External Call from MUMPS to translate between EBCDIC and ASCII 
*			S ASCII=$ZC("EBC2ASC",EBCDIC)
*			returns ASCII = ACSII translation of EBCDIC string	
*
* $Id: $
*
* $Log:	ebc2asc.c,v $
 * Revision 1.2  00/03/03  10:45:11  10:45:11  lyh ()
 * Replaced value for ` (grave character)
 * 
 * Revision 1.1  00/03/03  10:30:39  10:30:39  lyh ()
 * Initial revision
 * 
*
* $Revision: 1.2 $
*
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "extcall.h"


static char ebc_asc_table[]={
            0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
            0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
            0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18,
            0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
            0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28,
            0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
            0x2E, 0x2E, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
            0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x2E, 0x3F,
            0x20, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x2E, 0x2E, 0x2E, 0x3C, 0x28, 0x2B, 0x7C,
            0x26, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x2E, 0x21, 0x24, 0x2A, 0x29, 0x3B, 0x5E,
            0x2D, 0x2F, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x2E, 0x7C, 0x2C, 0x25, 0x5F, 0x3E, 0x3F,
            0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x60, 0x3A, 0x23, 0x40, 0x27, 0x3D, 0x22,
            0x2E, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
            0x69, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x2E, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71,
            0x72, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x2E, 0x7E, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79,
            0x7A, 0x2E, 0x2E, 0x2E, 0x5B, 0x2E, 0x2E,
            0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x2E, 0x2E, 0x2E, 0x2E, 0x5D, 0x2E, 0x2E,
            0x7B, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
            0x49, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x7D, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51,                                             
            0x52, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x5C, 0x2E, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59,
            0x5A, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E,
            0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,                 
            0x39, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E, 0x2E } ; 



void
ebc2asc(int count,STR_DESCRIPTOR *str_d,STR_DESCRIPTOR *str_d2,SLONG *rc)
{
	int i;
	char *str = (char *)NULL;
	char *ptr = (char *)NULL;

	if((str_d->str == (char *)NULL) || (str_d->length <= 0))
	{
		/*
		*	if Null String was passed
		*/
		str_d->length=0;
		*rc = MUMPS_FAILURE;
		return;
	}

	if((str = (char *)malloc(str_d->length)) == (char *)NULL)
	{
		*rc = MUMPS_FAILURE;
		return;
	}
	(void)memset(str,'\0',str_d->length);
	ptr = str;

	for(i=0;i<str_d->length;i++)
	{
		*ptr++ = ebc_asc_table[str_d->str[i]];
	}

	(void)memcpy(str_d2->str,str,str_d->length);
	str_d2->length = str_d->length;

	(void)free(str);

   	*rc = MUMPS_SUCCESS;
	return;
}

/* void
ebc2asc(int count,STR_DESCRIPTOR *str_d,SLONG *rc)
{
	int i;
	char *str = (char *)NULL;
	char *ptr = (char *)NULL;

	if((str_d->str == (char *)NULL) || (str_d->length <= 0))
	{ */
		/*
		*	if Null String was passed
		*/
	/*	str_d->length=0;
		*rc = MUMPS_FAILURE;
		return;
	}

	if((str = (char *)malloc(str_d->length)) == (char *)NULL)
	{
		*rc = MUMPS_FAILURE;
		return;
	}
	(void)memset(str,'\0',str_d->length);
	ptr = str;

	for(i=0;i<str_d->length;i++)
	{
		*ptr++ = ebc_asc_table[str_d->str[i]];
	}

	(void)memcpy(str_d->str,str,str_d->length);

	(void)free(str);

   	*rc = MUMPS_SUCCESS;
	return;
} */
